/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2010 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.folder.json;

import com.openexchange.ajax.customizer.folder.AdditionalFolderFieldList;

/**
 * {@link Constants} for the HTTP JSON interface of the folder component.
 * 
 * @author <a href="mailto:marcus.klein@open-xchange.com">Marcus Klein</a>
 * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
 */
public final class Constants {

    /**
     * The folder module identifier.
     */
    private static volatile String module = "folders";

    /**
     * The folder servlet path.
     */
    private static volatile String servletPath = "/ajax/" + module;

    /**
     * The list for additional folder fields.
     */
    public static final AdditionalFolderFieldList ADDITIONAL_FOLDER_FIELD_LIST = new AdditionalFolderFieldList();

    /**
     * No instantiation.
     */
    private Constants() {
        super();
    }

    /**
     * Gets the module string.
     * 
     * @return The module string
     */
    public static String getModule() {
        return module;
    }

    /**
     * Sets the module string.
     * 
     * @param module The module string
     */
    public static void setModule(final String module) {
        Constants.module = module;
        Constants.servletPath = "/ajax/" + module;
    }

    /**
     * Gets the servlet path.
     * 
     * @return The servlet path
     */
    public static String getServletPath() {
        return servletPath;
    }

    /**
     * Sets the servlet path.
     * 
     * @param servletPath The servlet path
     */
    public static void setServletPath(final String servletPath) {
        Constants.servletPath = servletPath;
    }

}
