/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2010 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.tools.iterator;

import com.openexchange.groupware.AbstractOXException;


/**
 * {@link FilteringSearchIterator}
 *
 * @author <a href="mailto:francisco.laguna@open-xchange.com">Francisco Laguna</a>
 */
public abstract class FilteringSearchIterator<T> implements SearchIterator<T>{

    private T next = null;
    private SearchIterator<T> delegate;
    
    public FilteringSearchIterator(SearchIterator<T> delegate) throws AbstractOXException {
        this.delegate = delegate;
        initNext();
    }
    
    /**
     * Test whether something should be included in the result or not
     * @param thing
     * @return
     * @throws AbstractOXException
     */
    public abstract boolean accept(T thing) throws AbstractOXException;
    
    
    public void addWarning(AbstractOXException warning) {
        delegate.addWarning(warning);
    }

    public void close() throws AbstractOXException {
        delegate.close();
    }

    public AbstractOXException[] getWarnings() {
        return delegate.getWarnings();
    }

    public boolean hasNext() throws AbstractOXException {
        return next != null;
    }

    public boolean hasWarnings() {
        return delegate.hasWarnings();
    }

    public T next() throws AbstractOXException {
        T current = next;
        initNext();
        return current;
    }

    public int size() {
        return -1;
    }
    
    protected void initNext() throws AbstractOXException {
        while(delegate.hasNext()) {
            next = delegate.next();
            if(accept(next)) {
                return;
            }
        }
        next = null;
    }
}
