/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2010 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.ajax.login;

import java.io.UnsupportedEncodingException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import javax.servlet.http.HttpServletRequest;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import com.openexchange.ajax.fields.Header;
import com.openexchange.ajax.fields.LoginFields;
import com.openexchange.config.ConfigurationService;
import com.openexchange.server.services.ServerServiceRegistry;
import com.openexchange.tools.encoding.Base64;


/**
 * {@link HashCalculator}
 *
 * @author <a href="mailto:francisco.laguna@open-xchange.com">Francisco Laguna</a>
 */
public class HashCalculator {
    private static final Log LOG = LogFactory.getLog(HashCalculator.class);
    
    public static String getHash(HttpServletRequest req) {
        try {
            MessageDigest md = MessageDigest.getInstance("MD5");
            md.update(getUserAgent(req).getBytes("UTF-8"));
            md.update(getClient(req).getBytes("UTF-8"));
            String fieldList = ServerServiceRegistry.getInstance().getService(ConfigurationService.class).getProperty(
                "com.openexchange.sessiond.hash.fields",
                "");
            String[] fields = fieldList.split("\\s*,\\s*");
            for (String field : fields) {
                String header = req.getHeader(field);
                if (header != null) {
                    md.update(header.getBytes("UTF-8"));
                }
            }

            byte[] digest = md.digest();

            return Base64.encode(digest).replaceAll("\\W", "");
        } catch (NoSuchAlgorithmException e) {
            LOG.fatal(e.getMessage(), e);
        } catch (UnsupportedEncodingException e) {
            LOG.fatal(e.getMessage(), e);
        }
        return "";
    }

    private static String getClient(HttpServletRequest req) {
        String parameter = req.getParameter(LoginFields.CLIENT_PARAM);
        if(parameter == null) {
            return "default";
        }
        return parameter;
    }

    private static String getUserAgent(HttpServletRequest req) {
        String header = req.getHeader(Header.USER_AGENT);
        if(header == null) {
            return "";
        }
        return header;
    }
}
