/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2010 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.context;

import java.util.List;
import com.openexchange.groupware.contexts.Context;
import com.openexchange.groupware.contexts.impl.ContextException;

/**
 * {@link ContextService} - Offers access method to context module.
 * 
 * @author <a href="mailto:thorben.betten@open-xchange.com">Thorben Betten</a>
 * 
 */
public interface ContextService {

    /**
     * Instantiates an implementation of the context interface and fill its
     * attributes according to the needs to be able to separate contexts.
     * 
     * @param loginContextInfo
     *            the login info for the context.
     * @return the unique identifier of the context or <code>-1</code> if no
     *         matching context exists.
     * @throws ContextException
     *             if an error occurs.
     */
    public abstract int getContextId(String loginContextInfo) throws ContextException;

    /**
     * Gets the context for the given context unique identifier.
     * 
     * @param contextId The unique identifier of the context.
     * @return The context
     * @throws ContextException If the specified context cannot be found or the update is running/started.
     */
    public Context getContext(int contextId) throws ContextException;

    /**
     * This method works like {@link #getContext(int)} but it does not give a {@link ContextException} if an update is running or must is
     * started.
     * @param contextId unique identifier of the context.
     * @return an implementation of the context or <code>null</code> if the context with the given identifier can't be found.
     * @throws ContextException if an error occurs.
     */
    public Context loadContext(int contextId) throws ContextException;

    /**
     * Invalidates the context object in cache(s).
     * 
     * @param contextId
     *            unique identifier of the context to invalidate
     * @throws ContextException
     * @throws ContextException
     *             if invalidating the context fails
     */
    public void invalidateContext(int contextId) throws ContextException;

    /**
     * Invalidates a login information in the cache.
     * 
     * @param loginContextInfo
     *            login information to invalidate.
     * @throws ContextException
     * @throws ContextException
     *             if invalidating the login information fails.
     */
    public void invalidateLoginInfo(String loginContextInfo) throws ContextException;

    /**
     * Gives a list of all context ids which are stored in the config database.
     * 
     * @return the list of context ids
     * @throws ContextException
     *             if reading the contexts fails.
     */
    public abstract List<Integer> getAllContextIds() throws ContextException;
}
