/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2010 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.event.impl;

import com.openexchange.groupware.tasks.Task;
import com.openexchange.session.Session;

/**
 * {@link TaskEventInterface}
 * 
 * @author <a href="mailto:sebastian.kauss@open-xchange.org">Sebastian Kauss</a>
 */
public interface TaskEventInterface {
    
    public void taskCreated(Task taskObj, Session sessionObj);
    
    public void taskModified(Task taskObj, Session sessionObj);

    /**
     * Invoked if one of task's participants changed his confirmation status to
     * accepted.
     * 
     * @param taskObj
     *            The task
     * @param sessionObj
     *            The user session
     */
    public void taskAccepted(Task taskObj, Session sessionObj);

    /**
     * Invoked if one of task's participants changed his confirmation status to
     * declined.
     * 
     * @param taskObj
     *            The task
     * @param sessionObj
     *            The user session
     */
    public void taskDeclined(Task taskObj, Session sessionObj);

    /**
     * Invoked if one of task's participants changed his confirmation status to
     * tentatively accepted
     * 
     * @param taskObj
     *            The task
     * @param sessionObj
     *            The user session
     */
    public void taskTentativelyAccepted(Task taskObj, Session sessionObj);
        
    public void taskDeleted(Task taskObj, Session sessionObj);
    
}
