/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2010 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
package com.openexchange.groupware.infostore;

import com.openexchange.api2.OXException;
import com.openexchange.database.provider.DBProvider;
import com.openexchange.database.provider.StaticDBPoolProvider;
import com.openexchange.groupware.container.FolderObject;
import com.openexchange.groupware.downgrade.DowngradeEvent;
import com.openexchange.groupware.downgrade.DowngradeFailedException;
import com.openexchange.groupware.downgrade.DowngradeListener;
import com.openexchange.groupware.infostore.facade.impl.InfostoreFacadeImpl;
import com.openexchange.tools.oxfolder.OXFolderAccess;
import com.openexchange.tools.session.ServerSession;
import com.openexchange.tools.session.ServerSessionAdapter;
import com.openexchange.tx.TransactionException;

/**
 * @author Francisco Laguna <francisco.laguna@open-xchange.com>
 */
public class InfostoreDowngrade extends DowngradeListener {
    @Override
	public void downgradePerformed(final DowngradeEvent event) throws DowngradeFailedException {
        final DBProvider provider = new StaticDBPoolProvider(event.getWriteCon());

        final ServerSession session = new ServerSessionAdapter(event.getSession(), event.getContext());

        final InfostoreFacade infostore = new InfostoreFacadeImpl(provider);
        infostore.setTransactional(true);
        infostore.setCommitsTransaction(false);
        try {

            final OXFolderAccess access = new OXFolderAccess(event.getContext());
            final FolderObject fo = access.getDefaultFolder(event.getNewUserConfiguration().getUserId(), FolderObject.INFOSTORE);
            final int folderId = fo.getObjectID();


            infostore.startTransaction();

            infostore.removeDocument(folderId, Long.MAX_VALUE, session);

            infostore.commit();
        } catch (final TransactionException e) {
            try {
                infostore.rollback();
            } catch (final TransactionException e1) {
                //IGNORE
            }
            throw new DowngradeFailedException(e);
        } catch (final OXException e) {
            try {
                infostore.rollback();
            } catch (final TransactionException e1) {
                //IGNORE
            }
            throw new DowngradeFailedException(e);
        } finally {
            try {
                infostore.finish();
            } catch (final TransactionException e) {
                //IGNORE
            }
        }
    }

    @Override
	public int getOrder() {
        return 2;
    }
}
