/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the Open-Xchange, Inc. group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2004-2010 Open-Xchange, Inc.
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.groupware.settings.tree.modules.mail;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import com.openexchange.groupware.contexts.Context;
import com.openexchange.groupware.ldap.User;
import com.openexchange.groupware.settings.IValueHandler;
import com.openexchange.groupware.settings.PreferencesItemService;
import com.openexchange.groupware.settings.ReadOnlyValue;
import com.openexchange.groupware.settings.Setting;
import com.openexchange.groupware.settings.SettingException;
import com.openexchange.groupware.userconfiguration.UserConfiguration;
import com.openexchange.mail.MailException;
import com.openexchange.mail.api.MailAccess;
import com.openexchange.mailaccount.MailAccountStorageService;
import com.openexchange.server.services.ServerServiceRegistry;
import com.openexchange.session.Session;

/**
 * Contains initialization for the modules configuration tree setting webmail.
 * @author <a href="mailto:marcus@open-xchange.org">Marcus Klein</a>
 */
public class Module implements PreferencesItemService {

    private static final Log LOG = LogFactory.getLog(Module.class);

    /**
     * Default constructor.
     */
    public Module() {
        super();
    }

    /**
     * {@inheritDoc}
     */
    public String[] getPath() {
        return new String[] { "modules", "mail", "module" };
    }

    /**
     * {@inheritDoc}
     */
    public IValueHandler getSharedValue() {
        return new ReadOnlyValue() {
            /**
             * {@inheritDoc}
             */
            public void getValue(final Session session, final Context ctx,
                final User user, final UserConfiguration userConfig,
                final Setting setting) throws SettingException {
                // Check if multiple mail accounts are enabled
                if (userConfig.isMultipleMailAccounts() && (ServerServiceRegistry.getInstance().getService(MailAccountStorageService.class) != null)) {
                    setting.setSingleValue(Boolean.TRUE);
                    return;
                }
                // Check if primary mail account is connectable
                MailAccess<?, ?> mail = null;
                try {
                    mail = MailAccess.getInstance(session);
                    mail.connect();
                    setting.setSingleValue(Boolean.TRUE);
                } catch (final MailException e) {
                    setting.setSingleValue(Boolean.FALSE);
                    LOG.error(e.getMessage(), e);
                } finally {
                    if (null != mail) {
                        mail.close(true);
                    }
                }
            }
            /**
             * {@inheritDoc}
             */
            public boolean isAvailable(final UserConfiguration userConfig) {
                return userConfig.hasWebMail();
            }
        };
    }
}
